﻿using MakeYourWeb.Data.Abstract;
using MakeYourWeb.Models.DataModels;
using MakeYourWeb.Models;
using Microsoft.EntityFrameworkCore;
using System;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;
using MkwSharedLibrary;
using MkwSharedLibrary.Models;
using MakeYourWeb.Services;

namespace MakeYourWeb.Data.Repositories
{
    public class ServerDataEditorRepository : BasicUserIncludedRepository, IServerDataEditor
    {
      
        public ServerDataEditorRepository(ApplicationDbContext context) : base(context)
        {
            //context binded inside parent - base constructor
        }

        public void DeleteSavedSite(UrlPageModel DeleteModel, string UsersIdentificator)
        {
            var User = base.GetUser(UsersIdentificator, IncludeSecret: false);

            var postCandidates = DbContext.Posts
                .Include(p => p.ConnectedWebsite)
                .Include(p=>p.ParentChannel)
                .Where(p => p.ConnectedWebsite.WebsiteURL.Equals(DeleteModel.Url) && p.ParentChannel.UserID.Equals(User.UserID));
            if (postCandidates.Any())
                DbContext.Posts.RemoveRange(postCandidates);

            var siteCandidates = DbContext.SavedWebsites.Where(c => c.UserID.Equals(UsersIdentificator) && c.WebSiteURL.Equals(DeleteModel.Url));
            if (siteCandidates.Any())
                DbContext.SavedWebsites.RemoveRange(siteCandidates);
        }

        public void AddPost(UrlPageModel website, string UsersIdentificator)
        {
            AppUser User = base.GetUser(IncludeSecret: false, UsersIdentificator: UsersIdentificator);

            bool isPostExist = DbContext.Posts
                .Include(p=>p.ConnectedWebsite)
                .Any(p => 
                        p.ChannelID.Equals(User.UsersChannel.ChannelID) && 
                        p.ConnectedWebsite.WebsiteURL.Equals(website.Url)
                        );

            if (!isPostExist)
            {
                Post NewPost = new Post()
                {
                    ChannelID = User.UsersChannel.ChannelID,
                    WebSiteURL = website.Url,
                    Created = DateTime.Now,
                    IsPublic = true, //TODO is post public property
                };

                DbContext.Posts.Add(NewPost); 
            }
        }

        public void AddWebsiteClick(UrlPageModel website, string UsersIdentificator)
        {
            var addedSite = DbContext.Websites.FirstOrDefault(s => s.WebsiteURL.Equals(website.Url));
            var user = base.GetUser(IncludeSecret: false, UsersIdentificator: UsersIdentificator);

            if (addedSite != null && user != null)
            {
                DbContext.WebsitesClicks.Add(new WebsiteClick()
                {
                    ClickedSite = addedSite,
                    Created = DateTime.Now,
                    UserID = UsersIdentificator,
                });
            }
        }

        private async Task<Website> ReturnWebsiteOrCreate(UrlPageModel UrlRelatedModel, IHtmlAnalyzeService HtmlService)
        {
            Uri siteUrl = new Uri(UrlRelatedModel.Url);
            bool ClearStatusCode = await Website.IsSiteStatusCodeOKAsync(siteUrl);

            WebSiteDM analyzedWebsite = await HtmlService.GetanAnalyzedModelAsync(UrlRelatedModel);

            WebPortal portal = this.SelectPortalOrCreate(siteUrl, analyzedWebsite, SaveChangesInsideMethod: false);
            Website newSite;
            if (!SiteRecordAlreadyExist(UrlRelatedModel.Url))
            {
                newSite = new Website()
                {
                    AddedDate = DateTime.Now,
                    WebsiteURL = UrlRelatedModel.Url,
                    Portal = portal,
                    Description = analyzedWebsite.Description,
                    Title = analyzedWebsite.Title,
                    FirstImage = analyzedWebsite.FirstImage,
                    MainHeader = analyzedWebsite.MainHeader,
                    MainParagraph = analyzedWebsite.MainParagraph,
                    SiteSaves = new List<SavedWebsite>(),
                    HasOkStatusCode = ClearStatusCode,
                    LastCheckDate = DateTime.Now,
                    PortalDomain = portal.PortalDomain
                };

                DbContext.Websites.Add(newSite);
                return newSite;
            }
            else return DbContext.Websites.Include(s => s.SiteSaves).First(s => s.WebsiteURL.Equals(UrlRelatedModel.Url));

        }


        public async Task AddWebsiteSaveAsync(UrlPageModel AddWebSiteModel, string UsersIdentificator, IHtmlAnalyzeService HtmlService)
        {

            bool SaveExist = DbContext.SavedWebsites.Any(c => c.UserID.Equals(UsersIdentificator) && c.WebSiteURL.Equals(AddWebSiteModel.Url));

            if (!SaveExist)
            {
                var user = base.GetUser(IncludeSecret: false, UsersIdentificator: UsersIdentificator);
                Website website = await ReturnWebsiteOrCreate(AddWebSiteModel, HtmlService);

                website.SiteSaves.Add(new SavedWebsite()
                {
                    UserID = UsersIdentificator,
                    WebSiteURL = website.WebsiteURL,
                    ConnectorCreated = DateTime.Now
                });
            }
        }



        private bool SiteRecordAlreadyExist(string wantedUrl) => DbContext.Websites.Any(s => s.WebsiteURL.Equals(wantedUrl));



        private WebPortal SelectPortalOrCreate(Uri websiteUri, WebSiteDM AnalyzedWebsite, bool SaveChangesInsideMethod)
        {
            var portalDomain = websiteUri.Authority;
            var portalMatch = DbContext.WebPortals.Where(p => p.PortalDomain.Equals(portalDomain));
            if (!portalMatch.Any())
            {
                var newPortal = new WebPortal()
                {
                    PortalDomain = AnalyzedWebsite.WebPortalName,
                    Image = AnalyzedWebsite.FirstImage,
                    Name = AnalyzedWebsite.WebPortalName,
                    Sites = new List<Website>()
                };

                DbContext.WebPortals.Add(newPortal);

                if (SaveChangesInsideMethod)
                    DbContext.SaveChanges();

                return newPortal;
            }

            return portalMatch.First();
        }

        public async Task<int> SaveDBAsync()
        {
            return await DbContext.SaveChangesAsync();
        }

        public void SubscribeChannel(SubscribeDM model)
        {
            var channel = DbContext.UsersChannels.Include(ch=>ch.Subscriptions).FirstOrDefault(ch => ch.ChannelID.Equals(model.ChannelIndetificator));

            if (channel.IsNull())
                return;

            if (!channel.Subscriptions.Any(s => s.UserID.Equals(model.UserIdentificator)))
            {
                channel.Subscriptions.Add(new Subscription()
                {
                    ChannelID = channel.ChannelID,
                    UserID = model.UserIdentificator,
                    SubscribedDate = DateTime.Now,
                });
            }
        }

        public void DeleteSubscription(SubscribeDM model)
        {
            var channel = DbContext.UsersChannels.Include(ch=>ch.Subscriptions).FirstOrDefault(ch => ch.ChannelID.Equals(model.ChannelIndetificator));

            if (channel.IsNull())
                return;

            if (DbContext.Subscriptions.Any(s => s.UserID.Equals(model.UserIdentificator)))
            {
                var DeletedSubscription = DbContext.Subscriptions.FirstOrDefault(s => s.ChannelID.Equals(model.ChannelIndetificator) && s.UserID.Equals(model.UserIdentificator));
                DbContext.Subscriptions.Remove(DeletedSubscription);
            }
        }

        public int SaveDB()
        {
            return DbContext.SaveChanges();
        }

        public void EditUserInformation(string UserIdentificator, UserInformationDM model)
        {
            AppUser user = DbContext.Users.First(u => u.UserID.Equals(UserIdentificator));
            user.Description = model.Description;
            user.Nick = model.Nick;
            DbContext.Entry(user).State = EntityState.Modified;           
        }
    }
}
